# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"update_templates": "00_actions.ipynb",
         "add_badges": "00_actions.ipynb",
         "install_git_filters": "00_actions.ipynb",
         "update_gitignore": "00_actions.ipynb",
         "update_project": "00_actions.ipynb",
         "sch_to_bom": "00_actions.ipynb",
         "bom_to_sch": "00_actions.ipynb",
         "export_manufacturing": "00_actions.ipynb",
         "export_sch": "00_actions.ipynb",
         "export_pcb": "00_actions.ipynb",
         "run_erc": "00_actions.ipynb",
         "run_drc": "00_actions.ipynb",
         "set_date": "00_actions.ipynb",
         "set_revision": "00_actions.ipynb",
         "test_erc": "01_test.ipynb",
         "test_drc": "01_test.ipynb",
         "validate_bom": "01_test.ipynb",
         "test_notebooks": "01_test.ipynb",
         "get_git_root": "02_utilities.ipynb",
         "setup_test_repo": "02_utilities.ipynb",
         "get_project_name": "02_utilities.ipynb",
         "get_project_metadata": "02_utilities.ipynb",
         "get_schematic_path": "02_utilities.ipynb",
         "get_bom_path": "02_utilities.ipynb",
         "get_board_path": "02_utilities.ipynb",
         "get_manufacturers": "02_utilities.ipynb",
         "get_gitignore_list": "02_utilities.ipynb",
         "in_gitignore": "02_utilities.ipynb",
         "run_docker_cmd": "02_utilities.ipynb",
         "run_kibot_docker": "02_utilities.ipynb",
         "get_board_metadata": "02_utilities.ipynb",
         "update_board_metadata": "02_utilities.ipynb",
         "get_schematic_metadata": "02_utilities.ipynb",
         "update_schematic_metadata": "02_utilities.ipynb",
         "github_badge": "02_utilities.ipynb",
         "kitspace_badge": "02_utilities.ipynb"}

modules = ["actions.py",
           "test.py",
           "utilities.py"]

doc_url = "https://ryanfobel.github.io/kicad-helpers/"

git_url = "https://github.com/ryanfobel/kicad-helpers/tree/main/"

def custom_doc_links(name): return None
